RaidInviteHelper = LibStub("AceAddon-3.0"):NewAddon("RaidInviteHelper", "AceTimer-3.0", "AceConsole-3.0", "AceEvent-3.0")

-- To Do:
-- Store group not working for other gorups.
-- Group 5 shows group 1.
-- 1 - Fix slash commands, make sure they're working properly.
-- 2 - Rework the store button and add a clear function, make store not destroy the list.
-- 3 - Add a help tab for more information.
-- 4 - Add a textbox to type in manually a players name/realm for invites.
-- 5 - Pipedream - add a feature showing if all players are online/offline.




-- Set all of the chat commands and make sure the databases are formed.
function RaidInviteHelper:OnInitialize()
	RaidInviteHelper:RegisterChatCommand("rihtest", "TestText")
	RaidInviteHelper:RegisterChatCommand("rihinvite", "Invite")
	RaidInviteHelper:RegisterChatCommand("rihstore", "Store")
	RaidInviteHelper:RegisterChatCommand("rihadd", "Add")
	RaidInviteHelper:RegisterChatCommand("rihremove", "Remove")
	RaidInviteHelper:RegisterChatCommand("rihmembers", "Members")
	RaidInviteHelper:RegisterChatCommand("rihhelp", "Help")
	RaidInviteHelper:RegisterChatCommand("rih", "Show")
	RaidInviteHelper:RegisterChatCommand("rl", "ReloadUI")
end


tableDB = {}
tableDB[1] =  'Group 1'
tableDB[2] =  'Group 2'
tableDB[3] =  'Group 3'
tableDB[4] =  'Group 4'
tableDB[5] =  'Group 5'

-- Sets the default table as table 1.
currentGroup = groupTable1


--Objective of this addon
-- 1) Store a list of player names, either by pulling from the current group or manual addition.
-- 2) Show an interface displaying the players names, as well as their online/offline status.
-- 3) Allow a button to click that sends an invite to all of the players on the list that are not in the party.
-- 4) Optional: Specify a specific time and date during which to automatically invite people.


-- Store all of the current party members into currentGroup
function RaidInviteHelper:Store()
	raidSize = GetNumGroupMembers();
	if raidSize == 0 then
		print("Not in a group. Moron.");
	else
		
		print("Storing the group.")
		for i = 1, raidSize do
			local name = GetRaidRosterInfo(i)
			
			print("Name = " .. name)
			-- The more variables you assigne to GetRaidRosterInfo the further in its list it goes, so if you want rank, the second setting, 
			-- you have to assign two variables and the second one will be rank.
			RaidInviteHelper:Add(name)
			
		end
			RaidInviteHelper:UpdateTables()
	end
end



-- Displays a list of all members of the current group.
function RaidInviteHelper:Members()
	rihEditBox:SetText("")
	--groupFontString:SetText("")
	--gFS = ""
	for i, v in ipairs(currentGroup) do 
	rihEditBox:SetText(rihEditBox:GetText() .. v .. "\n")
	--gFS = gFS .. v .. "\n"
	end
	--groupFontString:SetText(gFS)
	rihMemberDrop:SetList(currentGroup)
	end



-- Allows adding members to the current group manually.
function RaidInviteHelper:Add(memberAdd)
	isDupe = false;
	if memberAdd == nil or memberAdd == "" then
		print("No player selected.")
	else
		for i, v in ipairs(currentGroup) do 
			if v == memberAdd then 
			isDupe = true 
			print("Already a member.")
			end
		end
		if isDupe == false then
			
			table.insert(currentGroup, memberAdd);
			RaidInviteHelper:UpdateTables()
			print("Added ", memberAdd)
		end
	end
	
end

function RaidInviteHelper:UpdateTables()
	gNum = rihGroupDrop:GetValue()
	if gNum == 1 then
		groupTable1 = currentGroup	
	else if gNum == 2 then
		groupTable2 = currentGroup
	else if gNum == 3 then 
		groupTable3 = currentGroup
	else if gNum == 4 then
		groupTable4 = currentGroup
	else
		groupTable5 = currentGroup
end
end
end
end	
	RaidInviteHelper:Members()
end

-- Allows for the removal of a member of the currentGroup manually.
-- Also rearranges the table to not leave an empty space in it.
function RaidInviteHelper:Remove(memberRemove)
	for i, v in ipairs(currentGroup) do 
		if v == memberRemove then 
		table.remove(currentGroup, i)
		RaidInviteHelper:UpdateTables()
		print("Removed memberRemove")
		end
	end
end



-- Invites everyone in talbeGroupSet1. Has a wait function built in to help with not being able
-- to form a group as a raid group immediately.
function RaidInviteHelper:Invite()
	print("Invite")
	self.playerName = UnitName("player")
	self.timerCount = 0
	self.groupMems = 0
	self.tableSize = table.getn(currentGroup)
	self.tableIter = 1
	self.inviteTimer = self:ScheduleRepeatingTimer("InviteTime", .5)
end

-- A timer that helps deal with the lag of inviting and forming a raid group.
-- If you just iterate on the table it works so fast that WoW won't have time to conver it to a raid group.
-- This puts a .5 second delay on each invite and check to try and make up for that fact.
function RaidInviteHelper:InviteTime()
	self.currentPlayer = currentGroup[self.tableIter]
	if not UnitInParty(self.currentPlayer) then
		if self.currentPlayer ~= self.playerName then
			-- print("Inviting ", self.CurrentPlayer)
			InviteUnit(self.currentPlayer)
		end
	end
	if not IsInRaid(player) then
		ConvertToRaid()
	end
	self.tableIter = self.tableIter + 1
	if self.tableIter > self.tableSize then
		print("End of invites.")
		self:CancelTimer(self.inviteTimer)
	end
end

-- Help menu detailing the commands.

function RaidInviteHelper:Help(helpArg)
	if helpArg == "" then
		print("Help for Raid Invite Helper.")
		print("The following commands currently function.")
		print("/rihstore - Stores current party/raid in a table for use during invites. Type /rihhelp store for more help.")
		print("/rihinvite - Invites all characters stored. Type /rihhelp invite for more help.")
		print("/rihadd - Add a character manually, use /rihadd CharacterName-Server . Type /rihhelp add for more help.")
		print("/rihremove - Remove a character manually, use /rihremove CharacterName-Server . Type /rihhelp remove for more help.")
		print("/rihmembers - Shows all of the current characters in the saved group. Type /rihhelp members for more help.")
	else if helpArg == "store" then
		print("/rihstore - This command will store all of the current raid or group members in a table. The table will last between session. Each use of this OVERWRITES the prevoius table. It does not matter if the player is on another server.")
	else if helpArg == "invite" then
		print("/rihinvite - This command will invite everyone in the saved table. It will also convert the group to a raid in order to accomodate all of the users. It will only send out the invite if the player is not in the group.")
	else if helpArg == "add" then
		print("/rihadd CharacterName-Server - This command manually adds a character. If they are on your server, you do not have to include the -server tag. Will not take duplicate entries.")
		print("Example: /rihadd Kechlion-Maelstrom ")
	else if helpArg == "remove" then
		print("/rihremove CharacterName-Server - This command manually removes a character. This must match what is stored in the table or it will not work. Use the /rihmembers command to list all current members.")
		print("Example: /rihremove Kechlion")
	else if helpArg == "members" then
		print("/rihmembers - This command will list out the current population of characters that have been saved. Useful if you need to remove someone and are not sure what their name is.")
	else
		Print("Invalid option.")
	end
end
end
end
end
end
end

function RaidInviteHelper:ReloadUI()
	ReloadUI()
end

function RaidInviteHelper:ChangeGroup(gNum)
	if gNum == 1 then
		if groupTable1 == nil then groupTable1 = {} end
		currentGroup = groupTable1
	else if gNum == 2 then
		if groupTable2 == nil then groupTable2 = {} end
		currentGroup = groupTable2
	else if gNum == 3 then
		if groupTable3 == nil then groupTable3 = {} end
		currentGroup = groupTable3
	else if gNum == 4 then
		if groupTable4 == nil then groupTable4 = {} end
		currentGroup = groupTable4
	else
		if groupTable5 == nil then groupTable5 = {} end
		currentGroup = groupTable5
	end
	end
	end
	end
	RaidInviteHelper:Members()
end

function RaidInviteHelper:DeleteGroup(gtd)
	warningFrame = CreateFrame("Frame", "warningFrame")
	warningFrame:SetBackdrop({
		bgFile="Interface\\DialogFrame\\UI-DialogBox-Background", 
		edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border", 
		tile=1, tileSize=32, edgeSize=32, 
		insets={left=11, right=12, top=12, bottom=11}
	  })
	warningFrame:SetWidth(300)
	warningFrame:SetHeight(120)
	warningFrame:SetPoint("TOP", UIParnet, "TOP", 0, -40)
	
	warningFont = warningFrame:CreateFontString("warningFont", "Artwork", "GameFontNormal")
	warningFont:SetText("WARNING! This will delete Group " .. gtd .." Are you sure?")
	warningFont:SetPoint("Top", warningFrame, "Top", 0, -20)
	
	cancleBtn = CreateFrame("button", "cancelBtn", warningFrame, "UIPanelButtonTemplate")
	cancleBtn:SetHeight(36)
	cancleBtn:SetWidth(84)
	cancleBtn:SetPoint("BOTTOM", warningFrame, "BOTTOM", 0, 36)
	cancleBtn:SetText("Cancel")
	cancleBtn:SetScript("OnClick", function(sefl)
		PlaySound("igMainMenuOption") warningFrame:Hide() 
	end)
	
	warningBtn = CreateFrame("button","warningBtn", warningFrame, "UIPanelButtonTemplate")
	warningBtn:SetHeight(24)
	warningBtn:SetWidth(60)
	warningBtn:SetPoint("BOTTOM", warningFrame, "BOTTOM", 0, 10)
	warningBtn:SetText("DELETE")
	warningBtn:SetScript("OnClick", function(self) 
		PlaySound("igMainMenuOption") self:GetParent():Hide() 
		currentGroup = {}
		RaidInviteHelper:UpdateTables()
	end)
	
	
end

-- GUI TESTING

--[[
rihLuaFrame = CreateFrame("frame","rihLuaFrame")

rihLuaFrame:SetBackdrop({
      bgFile="Interface\\DialogFrame\\UI-DialogBox-Background", 
      edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border", 
      tile=1, tileSize=32, edgeSize=32, 
      insets={left=11, right=12, top=12, bottom=11}
})

rihLuaFrame:EnableKeyboard(true);
rihLuaFrame:SetScript("OnKeyDown",function(self,key)
--	This can be checked for the ESC key, but we'll check for what the keybind uses
	if GetBindingFromClick(key)=="TOGGLEGAMEMENU" then
		self:Hide();
	end
end);

-- Frame information.

rihLuaFrame:SetWidth(380)
rihLuaFrame:SetHeight(420)
rihLuaFrame:SetPoint("CENTER",UIParent)
rihLuaFrame:EnableMouse(true)
rihLuaFrame:SetMovable(true)
rihLuaFrame:RegisterForDrag("LeftButton")
rihLuaFrame:SetScript("OnDragStart", function(self) self:StartMoving() end)
rihLuaFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
rihLuaFrame:SetFrameStrata("HIGH")
rihLuaFrame:EnableKeyboard(false)


-- Font Strings

titleFontString = rihLuaFrame:CreateFontString("titleFontString", "Artwork", "GameFontNormal")
titleFontString:SetText("Raid Invite Helper")
titleFontString:SetPoint("TOP", rihLuaFrame, "TOP", 0, 10)

groupFontString = rihLuaFrame:CreateFontString("groupFontString", "Artwork","WhiteNormalNumberFont")
groupFontString:SetText("Member")
groupFontString:SetPoint("TOPLEFT", rihLuaFrame, "TOPLEFT", 20, -43)
groupFontString:SetJustifyH("LEFT")
groupFontString:SetJustifyV("TOP")


if not groupDropDown then
   CreateFrame("Button", "groupDropDown", rihLuaFrame, "UIDropDownMenuTemplate")
end
 
groupDropDown:ClearAllPoints()
groupDropDown:SetPoint("TOPLEFT", rihLuaFrame, "TOPLEFT", 5, -13.5)
groupDropDown:Show()

 
local items = {
   "Group 1",
   "Group 2",
   "Group 3",
   "Group 4",
   "Group 5"
}
 

local function OnClick(self)
   UIDropDownMenu_SetSelectedID(groupDropDown, self:GetID())
   print("Some Text " .. self:GetID())
   RaidInviteHelper:ChangeGroup(self:GetID())
end
 
function testPrint(arg)
	print(arg)
	print(groupDropDown:GetText())
end
 
local function initialize(self, level)
   local info = UIDropDownMenu_CreateInfo()
   for k,v in pairs(items) do
      info = UIDropDownMenu_CreateInfo()
      info.text = v
      info.value = k
      info.func = OnClick
      UIDropDownMenu_AddButton(info, level)
   end
end
 
 
UIDropDownMenu_Initialize(groupDropDown, initialize)
UIDropDownMenu_SetSelectedID(groupDropDown, 1)
UIDropDownMenu_JustifyText(groupDropDown, "CENTER")

-------------------------------------------





------------------------------------------------
--	Buttons	--
------------------------------------------------

button = CreateFrame("button","MyAddonButton", rihLuaFrame, "UIPanelButtonTemplate")
button:SetHeight(24)
button:SetWidth(60)
button:SetPoint("BOTTOM", rihLuaFrame, "BOTTOM", 0, 10)
button:SetText("Close")
button:SetScript("OnClick", function(self) PlaySound("igMainMenuOption") self:GetParent():Hide() end)

rihLuaStoreButton = CreateFrame("Button", "rihLuaStoreButton", rihLuaFrame, "UIPanelButtonTemplate")
rihLuaStoreButton:SetHeight(24)
rihLuaStoreButton:SetWidth(160)
rihLuaStoreButton:SetPoint("TOPRIGHT", rihLuaFrame, "TOPRIGHT", -30, -15)
rihLuaStoreButton:SetText("Store Current Group")




--]]



-- Gui portion of the addon. Yeah... it's basic. Deal with it.
function RaidInviteHelper:Show()


--frame:SetLayout("Custom_Layout")

-- Create the inital frame.
local AceGUI = LibStub("AceGUI-3.0")
-- https://www.wowace.com/addons/ace3/pages/ace-gui-3-0-tutorial/
rihFrame = AceGUI:Create("Frame")
rihFrame:SetTitle("Raid Invite Helper")
rihFrame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
rihFrame:SetLayout("Flow")
rihFrame:SetHeight(565)
rihFrame:SetWidth(620)
rihFrame:EnableResize("Hide")

-- Create a group for the text box to go in.
rihTextGroup = AceGUI:Create("SimpleGroup")
rihTextGroup:SetLayout("Flow")

--Create a group for the buttons to go in.
rihBtnGroup = AceGUI:Create("SimpleGroup")
rihBtnGroup:SetLayout("List")
rihBtnGroup:SetWidth(218)


rihFrame:AddChildren(rihTextGroup, rihBtnGroup)

-- A drop down box containing 5 different saved groups.
rihGroupDrop = AceGUI:Create("Dropdown")
rihGroupDrop:SetLabel("Groups")
rihGroupDrop:SetList(tableDB)
rihGroupDrop:SetValue(1)
rihGroupDrop:SetRelativeWidth(1)
rihGroupDrop:SetMultiselect(false)
rihGroupDrop:SetCallback("OnValueChanged", function()
	RaidInviteHelper:ChangeGroup(rihGroupDrop:GetValue())
end)
rihTextGroup:AddChild(rihGroupDrop)


-- An edit box to display all of the group members.
rihEditBox = AceGUI:Create("MultiLineEditBox")
rihEditBox:SetLabel("Raid Members")
rihEditBox:DisableButton(true)
rihEditBox:SetRelativeWidth(1)
rihEditBox:SetNumLines(30)
rihTextGroup:AddChild(rihEditBox)

-- A button to save the current group to the group members.
rihStoreBtn = AceGUI:Create("Button")
rihStoreBtn:SetText("Store Current Group")
rihStoreBtn:SetRelativeWidth(1)
rihStoreBtn:SetCallback("OnClick", function() RaidInviteHelper:Store() end)
rihBtnGroup:AddChild(rihStoreBtn)

-- A button to add a single person by name.
--rihManAddBtn = AceGUI:Create("Button")
--rihManAddBtn:SetText("Add Player")
--rihManAddBtn:SetRelativeWidth(1)
--rihManAddBtn:SetCallback("OnClick", function() RaidInviteHelper:Add(rihManualAdd.GetText()) end)
--rihBtnGroup:AddChild(rihManAddBtn)

-- Add a box for the user to type in a name of a person they would like to add to the group.
rihManualAdd = AceGUI:Create("EditBox")
rihManualAdd:SetLabel("Manually Add Player")
rihManualAdd:SetRelativeWidth(1)
rihManualAdd:SetCallback("OnEnterPressed", function() 
	RaidInviteHelper:Add(rihManualAdd:GetText())
	rihManualAdd:ClearFocus()
	rihManualAdd:SetText("")
end)
rihBtnGroup:AddChild(rihManualAdd)

-- A button to invite the current group members.
rihInviteBtn = AceGUI:Create("Button")
rihInviteBtn:SetText("Invite Group to Raid")
rihInviteBtn:SetRelativeWidth(1)
rihInviteBtn:SetCallback("OnClick", function() RaidInviteHelper:Invite() end)
rihBtnGroup:AddChild(rihInviteBtn)

-- A button to add a player.
rihAddBtn = AceGUI:Create("Button")
rihAddBtn:SetText("Add Target Player")
rihAddBtn:SetRelativeWidth(1)
rihAddBtn:SetCallback("OnClick", function()
	invitee = ""
	targName, targRealm = UnitName("target")
	if targName == nil then
		print("No Target")
	else if targRealm == nil then
		invitee = targName
		RaidInviteHelper:Add(invitee)
	else
		invitee = targName .. "-" .. targRealm
		RaidInviteHelper:Add(invitee)
	end
	end
end)
rihBtnGroup:AddChild(rihAddBtn)

-- A button to remove a character from the list.
rihRemoveBtn = AceGUI:Create("Button")
rihRemoveBtn:SetText("Remove from List")
rihRemoveBtn:SetRelativeWidth(1)
rihRemoveBtn:SetCallback("OnClick", function() 
	if rihMemberDrop:GetValue() ~= nil then
		print('Removing ' .. currentGroup[rihMemberDrop:GetValue()])
		table.remove(currentGroup, rihMemberDrop:GetValue())
		RaidInviteHelper:UpdateTables()
		rihMemberDrop:SetValue(nil)
	else
		print("No one selected.")
	end
end)
rihBtnGroup:AddChild(rihRemoveBtn)



-- A dropdownlist showing members of the current group -if populated-.
rihMemVar = ""
rihMemberDrop = AceGUI:Create("Dropdown")
rihMemberDrop:SetLabel("Members")
rihMemberDrop:SetRelativeWidth(1)
rihMemberDrop:SetMultiselect(false)
rihBtnGroup:AddChild(rihMemberDrop)
-- Running the following command will set the groups up.


-- A button to completely clear a given group.
rihGroupClear = AceGUI:Create("Button")
rihGroupClear:SetText("Delete Group")
rihGroupClear:SetRelativeWidth(1)
rihGroupClear:SetCallback("OnClick", function ()
	RaidInviteHelper:DeleteGroup(rihGroupDrop:GetValue())
end)
rihBtnGroup:AddChild(rihGroupClear)

RaidInviteHelper:ChangeGroup(1)
table.insert(UISpecialFrames, "rihFrame")
end
--End of file.
